@@ -27,7 +27,8 @@ githooks
 
 Some functions runned every each git action (usually before commit).
 
-Can derive `README.md` from `__init__.py` or generate rst files necessary for sphinx docs generator.
+Can derive run pytest tests, `README.md` from `__init__.py` or generate rst files necessary for sphinx docs generator.
+All with one line
 
 Check module docstrings for how to use it.
 
@@ -36,21 +37,29 @@ deploy
 
 Deploy app on Pypi.
 
+
+pyvueel
+-------
+Common functions for Python / Vue / Eel project.
+
+Run `mypythontools.pyvueel.help_starter_pack_vue_app()` for tutorial how to create such an app.
+
+
 misc
-------
-
-Miscellaneous. Set up root path if not cwd.
-
+----
+Miscellaneous
+Set up root path if not cwd for example.
 """
 
 from . import githooks
 from . import build
 from . import misc
+from . import pyvueel
 
-__version__ = "0.0.4"
+__version__ = "0.0.6"
 
 __author__ = "Daniel Malachov"
 __license__ = "MIT"
 __email__ = "malachovd@seznam.cz"
 
-__all__ = ['githooks', 'build', 'deploy', 'misc']
+__all__ = ['githooks', 'build', 'deploy', 'misc', 'pyvueel']
